﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using Appearance = System.Windows.Forms.Appearance;

namespace UltraGridTest
{
	public partial class Form1 : Form
	{
		private BindingList<IAufgabeInfo> m_taskList;
		private Dictionary<Guid, IAufgabeInfo> m_repository;

		public Form1()
		{
			InitializeComponent();
		}

		protected override void OnLoad(EventArgs e)
		{
			m_taskList = new BindingList<IAufgabeInfo>
			{
				new AufgabeInfo(Color.DarkGreen, "Aufgabe1", 1),
				new AufgabeInfo(Color.DarkBlue, "Aufgabe2", 3),
				new AufgabeInfo(Color.DarkOrange, "Aufgabe3", 3),
				new AufgabeInfo(Color.DarkCyan, "Aufgabe4", 1),
			};
			m_repository = m_taskList.ToDictionary(a => a.AufgabeGuid);
			fillTasks();
		}

		private void fillTasks()
		{
			ugTaskList.BeginUpdate();
			try
			{
				Refresh();

				//bool firstInit = (ugTaskList.DataSource == null);
				//if (!firstInit)
				//    gridStateMan.SaveUserSetting();

				//m_taskList = new BindingList<IAufgabeInfo>(
				//    ServiceLocator.Current.GetInstance<IDirectDatabaseMethods>()
				//        .GetAufgabenInfosCreatedByOrAssignedToUser((Guid)m_currentUser.ID)
				//        .OrderByDescending(x => x.CreateDate)
				//        .ToList());
				ugTaskList.DataSource = m_taskList;

				//filter();
				//gridStateMan.LoadUserSetting();

				//ugTaskList.ActivateSelectFirst();
			}
			finally
			{
				ugTaskList.EndUpdate();
			}
		}

		private void ugTaskList_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
		{
			var ci = System.Globalization.CultureInfo.CurrentCulture;
			e.Layout.Bands[0].Override.CellAppearance.TextTrimming = TextTrimming.EllipsisCharacter;
			e.Layout.Bands[0].Columns["ExpiredDateTime"].Style = Infragistics.Win.UltraWinGrid.ColumnStyle.Edit;
			e.Layout.Bands[0].Columns["ExpiredDateTime"].NullText = "Keine Angabe";
			e.Layout.Bands[0].Columns["ReminderDate"].Style = Infragistics.Win.UltraWinGrid.ColumnStyle.Edit;
			e.Layout.Bands[0].Columns["DoneDate"].Style = Infragistics.Win.UltraWinGrid.ColumnStyle.Edit;
			e.Layout.Bands[0].Columns["PriorityID"].Format = " ";
			e.Layout.Bands[0].Columns["Done"].Width = 10;
			e.Layout.Bands[0].Columns["Done"].Style = Infragistics.Win.UltraWinGrid.ColumnStyle.CheckBox;
			e.Layout.Bands[0].Columns["Done"].CellActivation = Activation.AllowEdit;
			e.Layout.Bands[0].Columns["Done"].CellClickAction = CellClickAction.Edit;

			e.Layout.Bands[0].AutoPreviewEnabled = true;
			e.Layout.Bands[0].AutoPreviewIndentation = 60;
			e.Layout.Bands[0].AutoPreviewField = "TaskDescription";

			//ugTaskList.SetOutlookAlikeLayout();
			//ugTaskList.SetColumnsOutlookDate("ddd " + ci.DateTimeFormat.ShortDatePattern, "ExpiredDateTime", "DoneDate", "ReminderDate");
		}

		private void ultraGrid1_InitializeRow(object sender, InitializeRowEventArgs e)
		{
			if (!e.Row.IsDataRow)
				return;
			initRowAppearance(e.Row);
		}

		private void initRowAppearance(UltraGridRow row)
		{
			var aufgabeInfo = (IAufgabeInfo)row.ListObject;
			if (aufgabeInfo == null)
				return;

			var rowAppearance = row.Appearance;
			rowAppearance.ImageBackgroundOrigin = ImageBackgroundOrigin.Container;
			rowAppearance.ImageBackgroundStyle = ImageBackgroundStyle.Tiled;
			rowAppearance.ImageVAlign = VAlign.Middle;
			rowAppearance.ImageHAlign = HAlign.Left;
			rowAppearance.FontData.Strikeout = (aufgabeInfo.Done ? DefaultableBoolean.True : DefaultableBoolean.False);

			var priorityAppearance = row.Cells["PriorityID"].Appearance;
			switch (aufgabeInfo.PriorityID)
			{
				case 1:
					replaceImage(priorityAppearance, Properties.Resources.PriorityLowArrow16x16);
					break;
				case 3:
					replaceImage(priorityAppearance, Properties.Resources.PriorityHighArrow16x16);
					break;
				default:
					replaceImage(priorityAppearance, null);
					break;
			}

			Infragistics.Win.Appearance actionsArtAppearance = row.Cells["AktionsArtID"].Appearance;
			replaceImage(actionsArtAppearance, getAktionsArtImage(aufgabeInfo.AktionsArtID));
		}

		private static Bitmap getAktionsArtImage(Color mdAktionsArt)
		{
			if (mdAktionsArt != Color.Transparent)
			{
				var img = new Bitmap(16, 16);
				using (Graphics g = Graphics.FromImage(img))
					g.FillRectangle(new SolidBrush(mdAktionsArt), 0, 0, 15, 15);
			}
			return null;
		}

		private static void replaceImage(Infragistics.Win.Appearance appearance, Image image)
		{
			if (appearance.Image != null)
			{
				// Dispose the last image
				var oldImage = appearance.Image as IDisposable;
				if (oldImage != null)
					oldImage.Dispose();
				appearance.Image = null;
			}
			appearance.Image = image;
		}


		private List<IAufgabeInfo> selectedAufgabenInfos
		{
			get
			{
				return ugTaskList.GetAllSelectedDataRows()
					.Select(x => (IAufgabeInfo)x.ListObject)
					.ToList();
			}
		}

		private void taskDoneToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var aufgaben = selectedAufgabenInfos;
			if (aufgaben.Count == 0)
				return;

			foreach (IAufgabeInfo aufgabeInfo in aufgaben)
			{
				var aufgabe = m_repository[aufgabeInfo.AufgabeGuid];
				aufgabe.Done = true;
				// aufgabe.Save();
			}
			reloadTasks(aufgaben);
		}

		private void reloadTasks(IEnumerable<IAufgabeInfo> aufgabenInfos)
		{
			//// Load everything and merge updated entries back so the grid doesn't change too much
			//// as it would when reloaeding all.
			//var refreshedInfosByGuid =
			//    ServiceLocator.Current.GetInstance<IDirectDatabaseMethods>()
			//        .GetAufgabenInfosCreatedByOrAssignedToUser((Guid)m_currentUser.ID)
			//        .ToDictionary(x => x.AufgabeGuid);

			foreach (var origAufInfo in aufgabenInfos)
			{
				var index = m_taskList.IndexOf(origAufInfo);

				IAufgabeInfo refreshedAufInfo;
				if (m_repository.TryGetValue(origAufInfo.AufgabeGuid, out refreshedAufInfo))
					m_taskList[index] = m_repository[origAufInfo.AufgabeGuid];
				else
					m_taskList.Remove(origAufInfo); // Task isn't there any more ... remove!
			}
		}
	}

	public class AufgabeInfo : IAufgabeInfo
	{
		public Guid AufgabeGuid { get; set; }
		public Color AktionsArtID { get; set; }
		public string Text { get; set; }
		public int PriorityID { get; set; }
		public bool Done { get; set; }

		public AufgabeInfo(Color taskID, string text, int priorityID) : this()
		{
			AktionsArtID = taskID;
			Text = text;
			PriorityID = priorityID;
		}

		private AufgabeInfo()
		{
			AufgabeGuid = Guid.NewGuid();
		}
	}
}
